﻿--{{{ Libraries

local RL = AceLibrary("Roster-2.1")
local L = AceLibrary("AceLocale-2.2"):new("GridStatusHots")

--}}}

local playerClass, englishClass = UnitClass("player")

local spellNameCache = {};
spellNameCache.Lifebloom = GetSpellInfo(33763);
spellNameCache.Regrowth = GetSpellInfo(8936);
spellNameCache.Rejuvenation = GetSpellInfo(774);
spellNameCache.Renew = GetSpellInfo(139);
spellNameCache.WildGrowth = GetSpellInfo(48438);
spellNameCache.SheathofLight = GetSpellInfo(54203);
spellNameCache.GlyphofFlashofLight = GetSpellInfo(54957);
spellNameCache.Riptide = GetSpellInfo(61295);
GridStatusHots = GridStatus:NewModule("GridStatusHots")
GridStatusHots.menuName = L["My HoTs"];

GridStatusHots.extraOptions = {
    ["frequency"] = {
		type = 'range',
		name = L["Refresh frequency"],
		desc = L["Seconds between status refreshes"],
		get = function() return GridStatusHots.db.profile.frequency end,
		set = function(v)
				  GridStatusHots.db.profile.frequency = v
				  GridStatusHots:UpdateFrequency()
			  end,
		min = 0.01,
		max = 1,
		step = 0.01,
		isPercent = false,
		order = -1,
	},
}


--{{{ AceDB defaults
--
GridStatusHots.defaultDB = {
	frequency = 0.5,
	alert_renew = {
		text = L["Buff: My Renew"],
		desc = L["Buff: My Renew"],
		enable = true,
		priority = 90,
		range = false,
		threshold2 = 4,
		threshold3 = 2,
		color = { r = 0, g = 1, b = 0, a = 1 },
		color2 = { r = 1, g = 1, b = 0, a = 1 },
		color3 = { r = 1, g = 0, b = 0, a = 1 },
	},
	alert_rejuv = {
		text = L["Buff: My Rejuvenation"],
		desc = L["Buff: My Rejuvenation"],
		enable = true,
		priority = 90,
		range = false,
		threshold2 = 4,
		threshold3 = 2,
		color = { r = 0, g = 0, b = 1, a = 1 },
		color2 = { r = 1, g = 1, b = 0, a = 1 },
		color3 = { r = 1, g = 0, b = 0, a = 1 },
	},
	alert_regrow = {
		text = L["Buff: My Regrowth"],
		desc = L["Buff: My Regrowth"],
		enable = true,
		priority = 90,
		range = false,
		threshold2 = 4,
		threshold3 = 2,
		color = { r = 1, g = 1, b = 1, a = 1 },
		color2 = { r = 1, g = 1, b = 0, a = 1 },
		color3 = { r = 1, g = 0, b = 0, a = 1 },
	},
	alert_lifebl = {
		text = L["Buff: My Lifebloom"],
		desc = L["Buff: My Lifebloom"],
		enable = true,
		priority = 95,
		range = false,
		threshold2 = 4,
		threshold3 = 2,
		color = { r = 1, g = 0, b = 0, a = 1 },
		color2 = { r = 1, g = 1, b = 0, a = 1 },
		color3 = { r = 0, g = 1, b = 0, a = 1 },
	},
	alert_lifebl_stack = {
		text = L["Buff: Lifebloom Stack Colored"],
		desc = L["Buff: Lifebloom Stack Colored"],
		enable = true,
		priority = 95,
		range = false,
		threshold2 = 4,
		threshold3 = 2,
		color = { r = 1, g = 0, b = 0, a = 1 },
		color2 = { r = 1, g = 1, b = 0, a = 1 },
		color3 = { r = 0, g = 1, b = 0, a = 1 },
	},
	alert_wgrowth = {
		text = L["Buff: My Wild Growth"],
		desc = L["Buff: My Wild Growth"],
		enable = true,
		priority = 90,
		range = false,
		threshold2 = 4,
		threshold3 = 2,
		color = { r = 0, g =1, b = 1, a = 1 },
		color2 = { r = 1, g = 1, b = 0, a = 1 },
		color3 = { r = 1, g = 0, b = 0, a = 1 },
	},
	alert_slight = {
		text = L["Buff: My Sheath of Light"],
		desc = L["Buff: My Sheath of Light"],
		enable = true,
		priority = 90,
		range = false,
		threshold2 = 4,
		threshold3 = 2,
		color = { r = 0, g = 1, b = 0, a = 1 },
		color2 = { r = 1, g = 1, b = 0, a = 1 },
		color3 = { r = 1, g = 0, b = 0, a = 1 },
	},
	alert_goflight = {
		text = L["Buff: My Glyph of Flash of Light"],
		desc = L["Buff: My Glyph of Flash of Light"],
		enable = true,
		priority = 90,
		range = false,
		threshold2 = 4,
		threshold3 = 2,
		color = { r = 0, g = 1, b = 0, a = 1 },
		color2 = { r = 1, g = 1, b = 0, a = 1 },
		color3 = { r = 1, g = 0, b = 0, a = 1 },
	},
	alert_riptide = {
		text = L["Buff: My Riptide"],
		desc = L["Buff: My Riptide"],
		enable = true,
		priority = 99,
		range = false,
		threshold2 = 4,
		threshold3 = 2,
		color = { r = 0, g = 1, b = 0, a = 1 },
		color2 = { r = 1, g = 1, b = 0, a = 1 },
		color3 = { r = 1, g = 0, b = 0, a = 1 },
	},
	alert_tothots = {
		text = L["Buff: Hot Count"],
		desc = L["Buff: Hot Count"],
		enable = true,
		priority = 97,
		range = false,
		color = { r = 0, g = 1, b = 0, a = 1 },
	},
}


local renew_hotcolors = {
	["threshold2"] = {
		type = "range",
		name = L["Threshold to activate color 2"],
		desc = L["Threshold to activate color 2"],
		max = 10,
		min = 1,
		step = .5,
		get = function ()
			      return GridStatusHots.db.profile.alert_renew.threshold2
		      end,
		set = function (v)
			       GridStatusHots.db.profile.alert_renew.threshold2 = v
		end,
	},
	["color2"] = {
		type = "color",
		name = L["Color 2"],
		desc = L["Color 2"],
		hasAlpha = true,
		get = function ()
			local color = GridStatusHots.db.profile.alert_renew.color2
			return color.r, color.g, color.b, color.a
		end,
		set = function (r, g, b, a)
			local color = GridStatusHots.db.profile.alert_renew.color2
			color.r = r
			color.g = g
			color.b = b
			color.a = a or 1
		end,
	},
	["threshold3"] = {
		type = "range",
		name = L["Threshold to activate color 3"],
		desc = L["Threshold to activate color 3"],
		max = 10,
		min = 1,
		step = .5,
		get = function ()
			      return GridStatusHots.db.profile.alert_renew.threshold3
		      end,
		set = function (v)
			       GridStatusHots.db.profile.alert_renew.threshold3 = v
		end,
	},
	["color3"] = {
		type = "color",
		name = L["Color 3"],
		desc = L["Color 3"],
		hasAlpha = true,
		get = function ()
			local color = GridStatusHots.db.profile.alert_renew.color3
			return color.r, color.g, color.b, color.a
		end,
		set = function (r, g, b, a)
			local color = GridStatusHots.db.profile.alert_renew.color3
			color.r = r
			color.g = g
			color.b = b
			color.a = a or 1
		end,
	},
}

local rejuv_hotcolors = {
	["threshold2"] = {
		type = "range",
		name = L["Threshold to activate color 2"],
		desc = L["Threshold to activate color 2"],
		max = 10,
		min = 1,
		step = .5,
		get = function ()
			      return GridStatusHots.db.profile.alert_rejuv.threshold2
		      end,
		set = function (v)
			       GridStatusHots.db.profile.alert_rejuv.threshold2 = v
		end,
	},
	["color2"] = {
		type = "color",
		name = L["Color 2"],
		desc = L["Color 2"],
		hasAlpha = true,
		get = function ()
			local color = GridStatusHots.db.profile.alert_rejuv.color2
			return color.r, color.g, color.b, color.a
		end,
		set = function (r, g, b, a)
			local color = GridStatusHots.db.profile.alert_rejuv.color2
			color.r = r
			color.g = g
			color.b = b
			color.a = a or 1
		end,
	},
	["threshold3"] = {
		type = "range",
		name = L["Threshold to activate color 3"],
		desc = L["Threshold to activate color 3"],
		max = 10,
		min = 1,
		step = .5,
		get = function ()
			      return GridStatusHots.db.profile.alert_rejuv.threshold3
		      end,
		set = function (v)
			       GridStatusHots.db.profile.alert_rejuv.threshold3 = v
		end,
	},
	["color3"] = {
		type = "color",
		name = L["Color 3"],
		desc = L["Color 3"],
		hasAlpha = true,
		get = function ()
			local color = GridStatusHots.db.profile.alert_rejuv.color3
			return color.r, color.g, color.b, color.a
		end,
		set = function (r, g, b, a)
			local color = GridStatusHots.db.profile.alert_rejuv.color3
			color.r = r
			color.g = g
			color.b = b
			color.a = a or 1
		end,
	},
}

local regrow_hotcolors = {
	["threshold2"] = {
		type = "range",
		name = L["Threshold to activate color 2"],
		desc = L["Threshold to activate color 2"],
		max = 10,
		min = 1,
		step = .5,
		get = function ()
			      return GridStatusHots.db.profile.alert_regrow.threshold2
		      end,
		set = function (v)
			       GridStatusHots.db.profile.alert_regrow.threshold2 = v
		end,
	},
	["color2"] = {
		type = "color",
		name = L["Color 2"],
		desc = L["Color 2"],
		hasAlpha = true,
		get = function ()
			local color = GridStatusHots.db.profile.alert_regrow.color2
			return color.r, color.g, color.b, color.a
		end,
		set = function (r, g, b, a)
			local color = GridStatusHots.db.profile.alert_regrow.color2
			color.r = r
			color.g = g
			color.b = b
			color.a = a or 1
		end,
	},
	["threshold3"] = {
		type = "range",
		name = L["Threshold to activate color 3"],
		desc = L["Threshold to activate color 3"],
		max = 10,
		min = 1,
		step = .5,
		get = function ()
			      return GridStatusHots.db.profile.alert_regrow.threshold3
		      end,
		set = function (v)
			       GridStatusHots.db.profile.alert_regrow.threshold3 = v
		end,
	},
	["color3"] = {
		type = "color",
		name = L["Color 3"],
		desc = L["Color 3"],
		hasAlpha = true,
		get = function ()
			local color = GridStatusHots.db.profile.alert_regrow.color3
			return color.r, color.g, color.b, color.a
		end,
		set = function (r, g, b, a)
			local color = GridStatusHots.db.profile.alert_regrow.color3
			color.r = r
			color.g = g
			color.b = b
			color.a = a or 1
		end,
	},
}

local lifebl_hotcolors = {
	["threshold2"] = {
		type = "range",
		name = L["Threshold to activate color 2"],
		desc = L["Threshold to activate color 2"],
		max = 10,
		min = 1,
		step = .5,
		get = function ()
			      return GridStatusHots.db.profile.alert_lifebl.threshold2
		      end,
		set = function (v)
			       GridStatusHots.db.profile.alert_lifebl.threshold2 = v
		end,
	},
	["color2"] = {
		type = "color",
		name = L["Color 2"],
		desc = L["Color 2"],
		hasAlpha = true,
		get = function ()
			local color = GridStatusHots.db.profile.alert_lifebl.color2
			return color.r, color.g, color.b, color.a
		end,
		set = function (r, g, b, a)
			local color = GridStatusHots.db.profile.alert_lifebl.color2
			color.r = r
			color.g = g
			color.b = b
			color.a = a or 1
		end,
	},
	["threshold3"] = {
		type = "range",
		name = L["Threshold to activate color 3"],
		desc = L["Threshold to activate color 3"],
		max = 10,
		min = 1,
		step = .5,
		get = function ()
			      return GridStatusHots.db.profile.alert_lifebl.threshold3
		      end,
		set = function (v)
			       GridStatusHots.db.profile.alert_lifebl.threshold3 = v
		end,
	},
	["color3"] = {
		type = "color",
		name = L["Color 3"],
		desc = L["Color 3"],
		hasAlpha = true,
		get = function ()
			local color = GridStatusHots.db.profile.alert_lifebl.color3
			return color.r, color.g, color.b, color.a
		end,
		set = function (r, g, b, a)
			local color = GridStatusHots.db.profile.alert_lifebl.color3
			color.r = r
			color.g = g
			color.b = b
			color.a = a or 1
		end,
	},
}

local lifebl_hotcolors_stack = {
	["color2"] = {
		type = "color",
		name = L["Color 2"],
		desc = L["Color 2"],
		hasAlpha = true,
		get = function ()
			local color = GridStatusHots.db.profile.alert_lifebl.color2
			return color.r, color.g, color.b, color.a
		end,
		set = function (r, g, b, a)
			local color = GridStatusHots.db.profile.alert_lifebl.color2
			color.r = r
			color.g = g
			color.b = b
			color.a = a or 1
		end,
	},
	["color3"] = {
		type = "color",
		name = L["Color 3"],
		desc = L["Color 3"],
		hasAlpha = true,
		get = function ()
			local color = GridStatusHots.db.profile.alert_lifebl.color3
			return color.r, color.g, color.b, color.a
		end,
		set = function (r, g, b, a)
			local color = GridStatusHots.db.profile.alert_lifebl.color3
			color.r = r
			color.g = g
			color.b = b
			color.a = a or 1
		end,
	},
}

local wgrowth_hotcolors = {
	["threshold2"] = {
		type = "range",
		name = L["Threshold to activate color 2"],
		desc = L["Threshold to activate color 2"],
		max = 10,
		min = 1,
		step = .5,
		get = function ()
			      return GridStatusHots.db.profile.alert_wgrowth.threshold2
		      end,
		set = function (v)
			       GridStatusHots.db.profile.alert_wgrowth.threshold2 = v
		end,
	},
	["color2"] = {
		type = "color",
		name = L["Color 2"],
		desc = L["Color 2"],
		hasAlpha = true,
		get = function ()
			local color = GridStatusHots.db.profile.alert_wgrowth.color2
			return color.r, color.g, color.b, color.a
		end,
		set = function (r, g, b, a)
			local color = GridStatusHots.db.profile.alert_wgrowth.color2
			color.r = r
			color.g = g
			color.b = b
			color.a = a or 1
		end,
	},
	["threshold3"] = {
		type = "range",
		name = L["Threshold to activate color 3"],
		desc = L["Threshold to activate color 3"],
		max = 10,
		min = 1,
		step = .5,
		get = function ()
			      return GridStatusHots.db.profile.alert_wgrowth.threshold3
		      end,
		set = function (v)
			       GridStatusHots.db.profile.alert_wgrowth.threshold3 = v
		end,
	},
	["color3"] = {
		type = "color",
		name = L["Color 3"],
		desc = L["Color 3"],
		hasAlpha = true,
		get = function ()
			local color = GridStatusHots.db.profile.alert_wgrowth.color3
			return color.r, color.g, color.b, color.a
		end,
		set = function (r, g, b, a)
			local color = GridStatusHots.db.profile.alert_wgrowth.color3
			color.r = r
			color.g = g
			color.b = b
			color.a = a or 1
		end,
	},
}

local slight_hotcolors = {
	["threshold2"] = {
		type = "range",
		name = L["Threshold to activate color 2"],
		desc = L["Threshold to activate color 2"],
		max = 10,
		min = 1,
		step = .5,
		get = function ()
			      return GridStatusHots.db.profile.alert_slight.threshold2
		      end,
		set = function (v)
			       GridStatusHots.db.profile.alert_slight.threshold2 = v
		end,
	},
	["color2"] = {
		type = "color",
		name = L["Color 2"],
		desc = L["Color 2"],
		hasAlpha = true,
		get = function ()
			local color = GridStatusHots.db.profile.alert_slight.color2
			return color.r, color.g, color.b, color.a
		end,
		set = function (r, g, b, a)
			local color = GridStatusHots.db.profile.alert_slight.color2
			color.r = r
			color.g = g
			color.b = b
			color.a = a or 1
		end,
	},
	["threshold3"] = {
		type = "range",
		name = L["Threshold to activate color 3"],
		desc = L["Threshold to activate color 3"],
		max = 10,
		min = 1,
		step = .5,
		get = function ()
			      return GridStatusHots.db.profile.alert_slight.threshold3
		      end,
		set = function (v)
			       GridStatusHots.db.profile.alert_slight.threshold3 = v
		end,
	},
	["color3"] = {
		type = "color",
		name = L["Color 3"],
		desc = L["Color 3"],
		hasAlpha = true,
		get = function ()
			local color = GridStatusHots.db.profile.alert_slight.color3
			return color.r, color.g, color.b, color.a
		end,
		set = function (r, g, b, a)
			local color = GridStatusHots.db.profile.alert_slight.color3
			color.r = r
			color.g = g
			color.b = b
			color.a = a or 1
		end,
	},
}

local goflight_hotcolors = {
	["threshold2"] = {
		type = "range",
		name = L["Threshold to activate color 2"],
		desc = L["Threshold to activate color 2"],
		max = 10,
		min = 1,
		step = .5,
		get = function ()
			      return GridStatusHots.db.profile.alert_goflight.threshold2
		      end,
		set = function (v)
			       GridStatusHots.db.profile.alert_goflight.threshold2 = v
		end,
	},
	["color2"] = {
		type = "color",
		name = L["Color 2"],
		desc = L["Color 2"],
		hasAlpha = true,
		get = function ()
			local color = GridStatusHots.db.profile.alert_goflight.color2
			return color.r, color.g, color.b, color.a
		end,
		set = function (r, g, b, a)
			local color = GridStatusHots.db.profile.alert_goflight.color2
			color.r = r
			color.g = g
			color.b = b
			color.a = a or 1
		end,
	},
	["threshold3"] = {
		type = "range",
		name = L["Threshold to activate color 3"],
		desc = L["Threshold to activate color 3"],
		max = 10,
		min = 1,
		step = .5,
		get = function ()
			      return GridStatusHots.db.profile.alert_goflight.threshold3
		      end,
		set = function (v)
			       GridStatusHots.db.profile.alert_goflight.threshold3 = v
		end,
	},
	["color3"] = {
		type = "color",
		name = L["Color 3"],
		desc = L["Color 3"],
		hasAlpha = true,
		get = function ()
			local color = GridStatusHots.db.profile.alert_goflight.color3
			return color.r, color.g, color.b, color.a
		end,
		set = function (r, g, b, a)
			local color = GridStatusHots.db.profile.alert_goflight.color3
			color.r = r
			color.g = g
			color.b = b
			color.a = a or 1
		end,
	},
}

local riptide_hotcolors = {
	["threshold2"] = {
		type = "range",
		name = L["Threshold to activate color 2"],
		desc = L["Threshold to activate color 2"],
		max = 10,
		min = 1,
		step = .5,
		get = function ()
			      return GridStatusHots.db.profile.alert_riptide.threshold2
		      end,
		set = function (v)
			       GridStatusHots.db.profile.alert_riptide.threshold2 = v
		end,
	},
	["color2"] = {
		type = "color",
		name = L["Color 2"],
		desc = L["Color 2"],
		hasAlpha = true,
		get = function ()
			local color = GridStatusHots.db.profile.alert_riptide.color2
			return color.r, color.g, color.b, color.a
		end,
		set = function (r, g, b, a)
			local color = GridStatusHots.db.profile.alert_riptide.color2
			color.r = r
			color.g = g
			color.b = b
			color.a = a or 1
		end,
	},
	["threshold3"] = {
		type = "range",
		name = L["Threshold to activate color 3"],
		desc = L["Threshold to activate color 3"],
		max = 10,
		min = 1,
		step = .5,
		get = function ()
			      return GridStatusHots.db.profile.alert_riptide.threshold3
		      end,
		set = function (v)
			       GridStatusHots.db.profile.alert_riptide.threshold3 = v
		end,
	},
	["color3"] = {
		type = "color",
		name = L["Color 3"],
		desc = L["Color 3"],
		hasAlpha = true,
		get = function ()
			local color = GridStatusHots.db.profile.alert_riptide.color3
			return color.r, color.g, color.b, color.a
		end,
		set = function (r, g, b, a)
			local color = GridStatusHots.db.profile.alert_riptide.color3
			color.r = r
			color.g = g
			color.b = b
			color.a = a or 1
		end,
	},
}
--}}}


function GridStatusHots:OnInitialize()
	self.super.OnInitialize(self)

	self:RegisterStatus("alert_tothots", L["Buff: Hot Count"], alert_tothots)
	if englishClass == "PRIEST" then
		self:RegisterStatus("alert_renew", L["Buff: My Renew"], renew_hotcolors)
	end
	if englishClass == "DRUID" then
		self:RegisterStatus("alert_rejuv", L["Buff: My Rejuvenation"], rejuv_hotcolors)
		self:RegisterStatus("alert_regrow", L["Buff: My Regrowth"], regrow_hotcolors)
		self:RegisterStatus("alert_lifebl", L["Buff: My Lifebloom"], lifebl_hotcolors)
		self:RegisterStatus("alert_lifebl_stack", L["Buff: Lifebloom Stack Colored"], lifebl_hotcolors_stack)
		self:RegisterStatus("alert_wgrowth", L["Buff: My Wild Growth"], wgrowth_hotcolors)
	end
	if englishClass == "PALADIN" then
		self:RegisterStatus("alert_slight", L["Buff: My Sheath of Light"], slight_hotcolors)
		self:RegisterStatus("alert_goflight", L["Buff: My Glyph of Flash of Light"], goflight_hotcolors)
	end
	if englishClass == "SHAMAN" then
		self:RegisterStatus("alert_riptide", L["Buff: My Riptide"], riptide_hotcolors)
	end
end

function GridStatusHots:OnEnable()
	local f = function()
		GridStatusHots:UpdateAll()
	end
	self:ScheduleRepeatingEvent("GHS_Refresh", self.UpdateAll, GridStatusHots.db.profile.frequency, self)
end

function GridStatusHots:UpdateFrequency()
	self:CancelScheduledEvent("GHS_Refresh");
	self:ScheduleRepeatingEvent("GHS_Refresh", self.UpdateAll, GridStatusHots.db.profile.frequency, self)
end

function GridStatusHots:Reset()
	self.super.Reset(self)
end

function GridStatusHots:UpdateAll()
	for u in RL:IterateRoster(true) do
		local name = UnitName(u.unitid) or ""
		local recnt,rgcnt,rjcnt,lbcnt,lbstack, wgcnt, slcnt, glcnt, ricnt = 0,0,0,0,0,0,0,0,0
		local retime,rjtime,rgtime,lbtime,wgtime,sltime,gltime,ritime,hcnttxt = nil,nil,nil,nil,nil,nil,nil,nil,nil

		-- Find number of hots pr unit
		for i=1,40 do -- this is not raid numbers, its possible buffs/debuffs to scan per member,
			local bname,brank,btexture,bcount,btype, bdur, bexptime, bismine = UnitBuff(u.unitid, i)
			if not bname then break end

			local btime = bexptime - GetTime();
			if bname == spellNameCache["Renew"] then
				recnt = recnt + 1;
				if (bismine and btime) then retime = btime end
			elseif bname == spellNameCache["Regrowth"] then
				rgcnt = rgcnt + 1;
				if (bismine and btime) then rgtime = btime end
			elseif bname == spellNameCache["Rejuvenation"] then
				rjcnt = rjcnt + 1;
				if (bismine and btime) then rjtime = btime end
			elseif bname == spellNameCache["Lifebloom"] then
				if (bismine and btime) then
					lbcnt = bcount;
					lbtime = btime
					lbstack = bcount
				end
			elseif bname == spellNameCache["WildGrowth"] then
				wgcnt = wgcnt + 1;
				if (bismine and btime) then wgtime = btime end
			elseif bname == spellNameCache["SheathofLight"] then
				slcnt = slcnt + 1;
				if (bismine and btime) then sltime = btime end
			elseif bname == spellNameCache["GlyphofFlashofLight"] then
				glcnt = glcnt + 1;
				if (bismine and btime) then gltime = btime end
			elseif bname == spellNameCache["Riptide"] then
				ricnt = ricnt + 1;
				if (bismine and btime) then ritime = btime end
			end
		end
		-- Set number of hots currently running on all units
		if recnt > 0 or rjcnt > 0 or rgcnt > 0 or lbcnt > 0 or wgcnt > 0 or slcnt > 0 or glcnt > 0 or ricnt > 0 then
			hcnttxt = tostring(recnt+rjcnt+rgcnt+lbcnt+wgcnt+slcnt+glcnt+ricnt)
			local settings = self.db.profile.alert_tothots
			self.core:SendStatusGained(name, "alert_tothots",
				settings.priority,
				(settings.range and 40),
				settings.color,
				tostring(hcnttxt)
			)
		else
			if self.core:GetCachedStatus(name, "alert_tothots") then self.core:SendStatusLost(name, "alert_tothots") end
		end

		--Renew
		if retime then
			-- Add self thrown countdown and status
			local settings = self.db.profile.alert_renew
			local hotcolor = settings.color
			if retime <= settings.threshold2 then hotcolor = settings.color2 end
			if retime <= settings.threshold3 then hotcolor = settings.color3 end
			self.core:SendStatusGained(name, "alert_renew",
				settings.priority,
				(settings.range and 40),
				hotcolor,
				tostring(math.floor(retime).."-"..tostring(hcnttxt))
			)
		else
			if self.core:GetCachedStatus(name, "alert_renew") then self.core:SendStatusLost(name, "alert_renew") end
		end

		--Regrowth
		if rgtime then
			-- Add self thrown countdown and status
			local settings = self.db.profile.alert_regrow
			local hotcolor = settings.color
			if rgtime <= settings.threshold2 then hotcolor = settings.color2 end
			if rgtime <= settings.threshold3 then hotcolor = settings.color3 end
			self.core:SendStatusGained(name, "alert_regrow",
				settings.priority,
				(settings.range and 40),
				hotcolor,
				tostring(math.floor(rgtime).."-"..tostring(hcnttxt))
			)
		else
			if self.core:GetCachedStatus(name, "alert_regrow") then self.core:SendStatusLost(name, "alert_regrow") end
		end

		--Rejuv
		if rjtime then
			-- Add self thrown countdown and status
			local settings = self.db.profile.alert_rejuv
			local hotcolor = settings.color
			if rjtime <= settings.threshold2 then hotcolor = settings.color2 end
			if rjtime <= settings.threshold3 then hotcolor = settings.color3 end
			self.core:SendStatusGained(name, "alert_rejuv",
				settings.priority,
				(settings.range and 40),
				hotcolor,
				tostring(math.floor(rjtime).."-"..tostring(hcnttxt))
			)
		else
			if self.core:GetCachedStatus(name, "alert_rejuv") then self.core:SendStatusLost(name, "alert_rejuv") end
		end

		--Lifebloom
		if lbtime then
			-- Add self thrown countdown and status
			local settings = self.db.profile.alert_lifebl_stack;
			local hotcolor = settings.color;

			if lbstack == 2 then hotcolor = settings.color2 end
			if lbstack == 3 then hotcolor = settings.color3 end
			self.core:SendStatusGained(name, "alert_lifebl_stack",
				settings.priority,
				(settings.range and 40),
				hotcolor,
				string.format("%d-%.1f", hcnttxt, lbtime)
			);

			local settings = self.db.profile.alert_lifebl;
			local hotcolor = settings.color;
			if lbtime <= settings.threshold2 then hotcolor = settings.color2 end
			if lbtime <= settings.threshold3 then hotcolor = settings.color3 end
			self.core:SendStatusGained(name, "alert_lifebl",
				settings.priority,
				(settings.range and 40),
				hotcolor,
				string.format("%d-%.1f-%d", lbcnt, lbtime, hcnttxt)
			)

		else
			if self.core:GetCachedStatus(name, "alert_lifebl") then self.core:SendStatusLost(name, "alert_lifebl") end
			if self.core:GetCachedStatus(name, "alert_lifebl_stack") then self.core:SendStatusLost(name, "alert_lifebl_stack") end
		end

		--Wild Growth
		if wgtime then
			local settings = self.db.profile.alert_wgrowth
			local hotcolor = settings.color
			if wgtime <= settings.threshold2 then hotcolor = settings.color2 end
			if wgtime <= settings.threshold3 then hotcolor = settings.color3 end
			self.core:SendStatusGained(name, "alert_wgrowth",
				settings.priority,
				(settings.range and 40),
				hotcolor,
				tostring(math.floor(wgtime).."-"..tostring(hcnttxt))
			)
		else
			if self.core:GetCachedStatus(name, "alert_wgrowth") then self.core:SendStatusLost(name, "alert_wgrowth") end
		end

		--Sheath of Light
		if sltime then
			local settings = self.db.profile.alert_slight
			local hotcolor = settings.color
			if sltime <= settings.threshold2 then hotcolor = settings.color2 end
			if sltime <= settings.threshold3 then hotcolor = settings.color3 end
			self.core:SendStatusGained(name, "alert_slight",
				settings.priority,
				(settings.range and 40),
				hotcolor,
				tostring(math.floor(sltime).."-"..tostring(hcnttxt))
			)
		else
			if self.core:GetCachedStatus(name, "alert_slight") then self.core:SendStatusLost(name, "alert_slight") end
		end

		--Glyph of Flash of Light
		if gltime then
			local settings = self.db.profile.alert_goflight
			local hotcolor = settings.color
			if gltime <= settings.threshold2 then hotcolor = settings.color2 end
			if gltime <= settings.threshold3 then hotcolor = settings.color3 end
			self.core:SendStatusGained(name, "alert_goflight",
				settings.priority,
				(settings.range and 40),
				hotcolor,
				tostring(math.floor(gltime).."-"..tostring(hcnttxt))
			)
		else
			if self.core:GetCachedStatus(name, "alert_goflight") then self.core:SendStatusLost(name, "alert_goflight") end
		end

		--Riptide
		if ritime then
			local settings = self.db.profile.alert_riptide
			local hotcolor = settings.color
			if ritime <= settings.threshold2 then hotcolor = settings.color2 end
			if ritime <= settings.threshold3 then hotcolor = settings.color3 end
			self.core:SendStatusGained(name, "alert_riptide",
				settings.priority,
				(settings.range and 40),
				hotcolor,
				tostring(math.floor(ritime).."-"..tostring(hcnttxt))
			)
		else
			if self.core:GetCachedStatus(name, "alert_riptide") then self.core:SendStatusLost(name, "alert_riptide") end
		end

	end
end
